@@ -102,6 +102,7 @@ def zones(self):
     self.add('AUTH', "zones", "healthchecks")
     self.add('AUTH', "zones", "healthchecks/preview")
     self.add('AUTH', "zones", "keyless_certificates")
+    self.add('AUTH', "zones", "origin_max_http_version")
     self.add('AUTH', "zones", "pagerules")
     self.add('AUTH', "zones", "pagerules/settings")
     self.add('AUTH', "zones", "purge_cache")
@@ -358,6 +359,7 @@ def accounts(self):
     self.add('AUTH', "accounts", "billing/profile")
 
     self.add('AUTH', 'accounts', 'cfd_tunnel')
+    self.add('AUTH', 'accounts', 'cfd_tunnel', 'configurations')
     self.add('AUTH', 'accounts', 'cfd_tunnel', 'connections')
     self.add('AUTH', 'accounts', 'cfd_tunnel', 'token')
 
@@ -409,6 +411,7 @@ def accounts(self):
 
     self.add('VOID', 'accounts', 'workers')
     self.add('AUTH', 'accounts', 'workers/account-settings')
+    self.add('AUTH', 'accounts', 'workers/domains')
     self.add('VOID', 'accounts', 'workers/durable_objects')
     self.add('AUTH', 'accounts', 'workers/durable_objects/namespaces')
     self.add('AUTH', 'accounts', 'workers/durable_objects/namespaces', 'objects')
@@ -508,9 +511,11 @@ def zones_access(self):
     self.add('AUTH', "zones", "access/apps")
     self.add('AUTH', "zones", "access/apps", "policies")
     self.add('AUTH', "zones", "access/apps", "revoke_tokens")
+    self.add('AUTH', 'zones', 'access/bookmarks')
     self.add('AUTH', "zones", "access/certificates")
 #   self.add('AUTH', "zones", "access/apps/ca")
     self.add('AUTH', "zones", "access/apps", "ca")
+    self.add('AUTH', "zones", "access/apps", "user_policy_checks")
     self.add('AUTH', "zones", "access/groups")
     self.add('AUTH', "zones", "access/identity_providers")
     self.add('AUTH', "zones", "access/organizations")
@@ -521,6 +526,7 @@ def accounts_access(self):
     """ API core commands for Cloudflare API"""
 
     self.add('VOID', "accounts", "access")
+    self.add('AUTH', 'accounts', 'access/bookmarks')
     self.add('AUTH', "accounts", "access/groups")
     self.add('AUTH', "accounts", "access/identity_providers")
     self.add('AUTH', "accounts", "access/organizations")
@@ -592,6 +598,7 @@ def accounts_extras(self):
 
     self.add('AUTH', 'accounts', 'gateway')
     self.add('AUTH', 'accounts', 'gateway/app_types')
+    self.add('AUTH', 'accounts', 'gateway/categories')
     self.add('AUTH', 'accounts', 'gateway/configuration')
     self.add('AUTH', 'accounts', 'gateway/lists')
     self.add('AUTH', 'accounts', 'gateway/lists', 'items')
@@ -612,6 +619,8 @@ def accounts_extras(self):
     self.add('VOID', 'accounts', 'intel')
     self.add('VOID', 'accounts', 'intel-phishing')
     self.add('AUTH', 'accounts', 'intel-phishing/predict')
+    self.add('AUTH', 'accounts', 'intel/asn')
+    self.add('AUTH', 'accounts', 'intel/asn', 'subnets')
     self.add('AUTH', 'accounts', 'intel/dns')
     self.add('AUTH', 'accounts', 'intel/domain')
     self.add('AUTH', 'accounts', 'intel/domain-history')
@@ -637,6 +646,8 @@ def accounts_extras(self):
 
     self.add('AUTH', 'accounts', 'pcaps')
     self.add('AUTH', 'accounts', 'pcaps', 'download')
+    self.add('AUTH', 'accounts', 'pcaps/ownership')
+    self.add('AUTH', 'accounts', 'pcaps/ownership/validate')
 
     self.add('VOID', 'accounts', 'teamnet')
     self.add('AUTH', 'accounts', 'teamnet/routes')
@@ -652,6 +663,7 @@ def zones_extras(self):
     self.add('AUTH', 'zones', 'cache/variants')
     self.add('AUTH', 'zones', 'managed_headers')
     self.add('AUTH', 'zones', 'rulesets')
+    self.add('AUTH', 'zones', 'rulesets', 'rules')
     self.add('AUTH', 'zones', 'rulesets', 'versions')
     self.add('VOID', 'zones', 'rulesets/phases')
     self.add('AUTH', 'zones', 'rulesets/phases', 'entrypoint')
@@ -661,6 +673,8 @@ def zones_extras(self):
     self.add('AUTH', 'zones', 'settings/automatic_platform_optimization')
     self.add('AUTH', 'zones', 'settings/orange_to_orange')
     self.add('AUTH', 'zones', 'url_normalization')
+    self.add('VOID', 'zones', 'web3')
+    self.add('AUTH', 'zones', 'web3/hostnames')
 
 def accounts_email(self):
     """ accounts email """
