@@ -2,11 +2,12 @@
 from __future__ import absolute_import
 
 import json
+import keyword
 import requests
 
 from .network import CFnetwork
 from .logging_helper import CFlogger
-from .utils import user_agent, sanitize_secrets
+from .utils import user_agent, build_curl
 from .read_configs import read_configs
 from .api_v4 import api_v4
 from .api_extras import api_extras
@@ -48,20 +49,32 @@ class CloudFlare(object):
             self.network = CFnetwork(use_sessions=self.use_sessions)
             self.user_agent = user_agent()
 
+            ## We don't need to check this here as we test for
+            ## this when building the authentication headers
+            ##
+            ##if not isinstance(self.email, str):
+            ##    raise ValueError('email argument not string')
+            ##if not isinstance(self.token, str):
+            ##    raise ValueError('token argument not string')
+            ##if not isinstance(self.certtoken, str):
+            ##    raise ValueError('certtoken argument not string')
+            ##if not isinstance(self.base_url, str):
+            ##    raise ValueError('base url argument not string')
+
             if 'debug' in config and config['debug']:
                 self.logger = CFlogger(config['debug']).getLogger()
             else:
                 self.logger = None
 
         def call_with_no_auth(self, method, parts,
-                              identifier1=None, identifier2=None, identifier3=None,
+                              identifier1=None, identifier2=None, identifier3=None, identifier4=None,
                               params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
             headers = {}
             self._AddHeaders(headers)
             return self._call(method, headers, parts,
-                              identifier1, identifier2, identifier3,
+                              identifier1, identifier2, identifier3, identifier4,
                               params, data, files)
 
         def _AddHeaders(self, headers):
@@ -108,7 +121,7 @@ class CloudFlare(object):
             headers['X-Auth-User-Service-Key'] = certtoken
 
         def call_with_auth(self, method, parts,
-                           identifier1=None, identifier2=None, identifier3=None,
+                           identifier1=None, identifier2=None, identifier3=None, identifier4=None,
                            params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
@@ -124,11 +137,11 @@ class CloudFlare(object):
                 # however something isn't right and this works ... look at again later!
                 del headers['Content-Type']
             return self._call(method, headers, parts,
-                              identifier1, identifier2, identifier3,
+                              identifier1, identifier2, identifier3, identifier4,
                               params, data, files)
 
         def call_with_auth_unwrapped(self, method, parts,
-                                     identifier1=None, identifier2=None, identifier3=None,
+                                     identifier1=None, identifier2=None, identifier3=None, identifier4=None,
                                      params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
@@ -144,11 +157,11 @@ class CloudFlare(object):
                 # however something isn't right and this works ... look at again later!
                 del headers['Content-Type']
             return self._call_unwrapped(method, headers, parts,
-                                        identifier1, identifier2, identifier3,
+                                        identifier1, identifier2, identifier3, identifier4,
                                         params, data, files)
 
         def call_with_certauth(self, method, parts,
-                               identifier1=None, identifier2=None, identifier3=None,
+                               identifier1=None, identifier2=None, identifier3=None, identifier4=None,
                                params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
@@ -156,37 +169,14 @@ class CloudFlare(object):
             self._AddHeaders(headers)
             self._AddCerttokenHeaders(headers, method)
             return self._call(method, headers, parts,
-                              identifier1, identifier2, identifier3,
+                              identifier1, identifier2, identifier3, identifier4,
                               params, data, files)
 
         def _call_network(self, method, headers, parts,
-                     identifier1=None, identifier2=None, identifier3=None,
+                     identifier1=None, identifier2=None, identifier3=None, identifier4=None,
                      params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
-            if self.logger:
-                self.logger.debug('Call: %s,%s,%s,%s,%s,%s',
-                                  str(parts[0]),
-                                  str(identifier1),
-                                  str(parts[1]),
-                                  str(identifier2),
-                                  str(parts[2]),
-                                  str(identifier3))
-                if params is not None:
-                    try:
-                        str_params = json.dumps(params)
-                    except:
-                        str_params = str(params)
-                    self.logger.debug('Call: params=%s', str_params)
-                if data is not None:
-                    try:
-                        str_data = json.dumps(data)
-                    except:
-                        str_data = str(data)
-                    self.logger.debug('Call: data=%s', str_data)
-                if files is not None:
-                    self.logger.debug('Call: upload file %r', files)
-
             if (method is None) or (parts[0] is None):
                 # should never happen
                 raise CloudFlareInternalError(0, 'You must specify a method and endpoint')
@@ -219,22 +209,27 @@ class CloudFlare(object):
                 url += '/' + identifier3
 
             if self.logger:
-                self.logger.debug('Call: method=%s url=%s', str(method), str(url))
-                self.logger.debug('Call: headers=%s', str(sanitize_secrets(headers)))
+                msg = build_curl(method, url, headers, params, data, files)
+                self.logger.debug('Call: emulated curl command ...\n' + msg)
 
             try:
-                if self.logger:
-                    self.logger.debug('Call: doit!')
                 response = self.network(method, url, headers, params, data, files)
+            except requests.RequestException as e:
                 if self.logger:
-                    self.logger.debug('Call: done!')
+                    self.logger.debug('Call: requests exception! "%s"' % (e))
+                raise CloudFlareAPIError(0, e)
+            except requests.ConnectionError as e:
+                if self.logger:
+                    self.logger.debug('Call: requests connection exception! "%s"' % (e))
+                raise CloudFlareAPIError(0, 'connection error')
+            except requests.exceptions.Timeout as e:
+                if self.logger:
+                    self.logger.debug('Call: requests timeout exception! "%s"' % (e))
+                raise CloudFlareAPIError(0, 'connection timeout')
             except Exception as e:
                 if self.logger:
                     self.logger.debug('Call: exception! "%s"' % (e))
-                raise CloudFlareAPIError(0, 'connection failed.')
-
-            if self.logger:
-                self.logger.debug('Response: url=%s', response.url)
+                raise
 
             # Create response_{type|code|data}
             try:
@@ -298,17 +293,15 @@ class CloudFlare(object):
             return [response_type, response_code, response_data]
 
         def _raw(self, method, headers, parts,
-                 identifier1=None, identifier2=None, identifier3=None,
+                 identifier1=None, identifier2=None, identifier3=None, identifier4=None,
                  params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
             [response_type, response_code, response_data] = self._call_network(method,
                                                                                headers, parts,
-                                                                               identifier1,
-                                                                               identifier2,
-                                                                               identifier3,
+                                                                               identifier1, identifier2, identifier3, identifier4,
                                                                                params, data, files)
-     
+
             if response_type == 'application/json':
                 # API says it's JSON; so it better be parsable as JSON
                 # NDJSON is returned by Enterprise Log Share i.e. /zones/:id/logs/received
@@ -450,12 +443,12 @@ class CloudFlare(object):
             return response_data
 
         def _call(self, method, headers, parts,
-                  identifier1=None, identifier2=None, identifier3=None,
+                  identifier1=None, identifier2=None, identifier3=None, identifier4=None,
                   params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
             response_data = self._raw(method, headers, parts,
-                                      identifier1, identifier2, identifier3,
+                                      identifier1, identifier2, identifier3, identifier4,
                                       params, data, files)
 
             # Sanatize the returned results - just in case API is messed up
@@ -500,7 +493,7 @@ class CloudFlare(object):
                         response_data['success'] = True
 
             if response_data['success'] is False:
-                if 'errors' in response_data:
+                if 'errors' in response_data and response_data['errors'] != None:
                     errors = response_data['errors'][0]
                 else:
                     errors = {}
@@ -553,12 +546,12 @@ class CloudFlare(object):
             return result
 
         def _call_unwrapped(self, method, headers, parts,
-                            identifier1=None, identifier2=None, identifier3=None,
+                            identifier1=None, identifier2=None, identifier3=None, identifier4=None,
                             params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
             response_data = self._raw(method, headers, parts,
-                                      identifier1, identifier2, identifier3,
+                                      identifier1, identifier2, identifier3, identifier4,
                                       params, data, files)
             if self.logger:
                 self.logger.debug('Response: %s', response_data)
@@ -587,44 +580,44 @@ class CloudFlare(object):
     class _AddUnused(object):
         """ Cloudflare v4 API"""
 
-        def __init__(self, base, p1, p2=None, p3=None):
+        def __init__(self, base, p1, p2=None, p3=None, p4=None, p5=None):
             """ Cloudflare v4 API"""
 
             self._base = base
-            self._parts_unused = [p1, p2, p3]
+            self._parts_unused = [p1, p2, p3, p4, p5]
 
-        def __call__(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def __call__(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             # This is the same as a get()
-            return self.get(identifier1, identifier2, identifier3, params, data)
+            return self.get(identifier1, identifier2, identifier3, identifier4, params=params, data=data)
 
         def __str__(self):
             """ Cloudflare v4 API"""
 
             return '[%s]' % ('/' + '/:id/'.join(filter(None, self._parts_unused)))
 
-        def get(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def get(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             raise CloudFlareAPIError(0, 'get() call not available for this endpoint')
 
-        def patch(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def patch(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             raise CloudFlareAPIError(0, 'patch() call not available for this endpoint')
 
-        def post(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None, files=None):
+        def post(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
             raise CloudFlareAPIError(0, 'post() call not available for this endpoint')
 
-        def put(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def put(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             raise CloudFlareAPIError(0, 'put() call not available for this endpoint')
 
-        def delete(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def delete(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             raise CloudFlareAPIError(0, 'delete() call not available for this endpoint')
@@ -632,46 +625,46 @@ class CloudFlare(object):
     class _AddNoAuth(object):
         """ Cloudflare v4 API"""
 
-        def __init__(self, base, p1, p2=None, p3=None):
+        def __init__(self, base, p1, p2=None, p3=None, p4=None, p5=None):
             """ Cloudflare v4 API"""
 
             self._base = base
-            self._parts = [p1, p2, p3]
+            self._parts = [p1, p2, p3, p4, p5]
 
-        def __call__(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def __call__(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             # This is the same as a get()
-            return self.get(identifier1, identifier2, identifier3, params, data)
+            return self.get(identifier1, identifier2, identifier3, identifier4, params=params, data=data)
 
         def __str__(self):
             """ Cloudflare v4 API"""
 
             return '[%s]' % ('/' + '/:id/'.join(filter(None, self._parts)))
 
-        def get(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def get(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_no_auth('GET', self._parts,
-                                                identifier1, identifier2, identifier3,
+                                                identifier1, identifier2, identifier3, identifier4,
                                                 params, data)
 
-        def patch(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def patch(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             raise CloudFlareAPIError(0, 'patch() call not available for this endpoint')
 
-        def post(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None, files=None):
+        def post(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
             raise CloudFlareAPIError(0, 'post() call not available for this endpoint')
 
-        def put(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def put(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             raise CloudFlareAPIError(0, 'put() call not available for this endpoint')
 
-        def delete(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def delete(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             raise CloudFlareAPIError(0, 'delete() call not available for this endpoint')
@@ -679,169 +672,169 @@ class CloudFlare(object):
     class _AddWithAuth(object):
         """ Cloudflare v4 API"""
 
-        def __init__(self, base, p1, p2=None, p3=None):
+        def __init__(self, base, p1, p2=None, p3=None, p4=None, p5=None):
             """ Cloudflare v4 API"""
 
             self._base = base
-            self._parts = [p1, p2, p3]
+            self._parts = [p1, p2, p3, p4, p5]
 
-        def __call__(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def __call__(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             # This is the same as a get()
-            return self.get(identifier1, identifier2, identifier3, params, data)
+            return self.get(identifier1, identifier2, identifier3, identifier4, params=params, data=data)
 
         def __str__(self):
             """ Cloudflare v4 API"""
 
             return '[%s]' % ('/' + '/:id/'.join(filter(None, self._parts)))
 
-        def get(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def get(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_auth('GET', self._parts,
-                                             identifier1, identifier2, identifier3,
+                                             identifier1, identifier2, identifier3, identifier4,
                                              params, data)
 
-        def patch(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def patch(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_auth('PATCH', self._parts,
-                                             identifier1, identifier2, identifier3,
+                                             identifier1, identifier2, identifier3, identifier4,
                                              params, data)
 
-        def post(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None, files=None):
+        def post(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_auth('POST', self._parts,
-                                             identifier1, identifier2, identifier3,
+                                             identifier1, identifier2, identifier3, identifier4,
                                              params, data, files)
 
-        def put(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def put(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_auth('PUT', self._parts,
-                                             identifier1, identifier2, identifier3,
+                                             identifier1, identifier2, identifier3, identifier4,
                                              params, data)
 
-        def delete(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def delete(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_auth('DELETE', self._parts,
-                                             identifier1, identifier2, identifier3,
+                                             identifier1, identifier2, identifier3, identifier4,
                                              params, data)
 
     class _AddWithAuthUnwrapped(object):
         """ Cloudflare v4 API"""
 
-        def __init__(self, base, p1, p2=None, p3=None):
+        def __init__(self, base, p1, p2=None, p3=None, p4=None, p5=None):
             """ Cloudflare v4 API"""
 
             self._base = base
-            self._parts = [p1, p2, p3]
+            self._parts = [p1, p2, p3, p4, p5]
 
-        def __call__(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def __call__(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             # This is the same as a get()
-            return self.get(identifier1, identifier2, identifier3, params, data)
+            return self.get(identifier1, identifier2, identifier3, identifier4, params=params, data=data)
 
         def __str__(self):
             """ Cloudflare v4 API"""
 
             return '[%s]' % ('/' + '/:id/'.join(filter(None, self._parts)))
 
-        def get(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def get(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_auth_unwrapped('GET', self._parts,
-                                                       identifier1, identifier2, identifier3,
+                                                       identifier1, identifier2, identifier3, identifier4,
                                                        params, data)
 
-        def patch(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def patch(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_auth_unwrapped('PATCH', self._parts,
-                                                       identifier1, identifier2, identifier3,
+                                                       identifier1, identifier2, identifier3, identifier4,
                                                        params, data)
 
-        def post(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None, files=None):
+        def post(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_auth_unwrapped('POST', self._parts,
-                                                       identifier1, identifier2, identifier3,
+                                                       identifier1, identifier2, identifier3, identifier4,
                                                        params, data, files)
 
-        def put(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def put(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_auth_unwrapped('PUT', self._parts,
-                                                       identifier1, identifier2, identifier3,
+                                                       identifier1, identifier2, identifier3, identifier4,
                                                        params, data)
 
-        def delete(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def delete(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_auth_unwrapped('DELETE', self._parts,
-                                                       identifier1, identifier2, identifier3,
+                                                       identifier1, identifier2, identifier3, identifier4,
                                                        params, data)
 
     class _AddWithCertAuth(object):
         """ Cloudflare v4 API"""
 
-        def __init__(self, base, p1, p2=None, p3=None):
+        def __init__(self, base, p1, p2=None, p3=None, p4=None, p5=None):
             """ Cloudflare v4 API"""
 
             self._base = base
-            self._parts = [p1, p2, p3]
+            self._parts = [p1, p2, p3, p4, p5]
 
-        def __call__(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def __call__(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             # This is the same as a get()
-            return self.get(identifier1, identifier2, identifier3, params, data)
+            return self.get(identifier1, identifier2, identifier3, identifier4, params=params, data=data)
 
         def __str__(self):
             """ Cloudflare v4 API"""
 
             return '[%s]' % ('/' + '/:id/'.join(filter(None, self._parts)))
 
-        def get(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def get(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_certauth('GET', self._parts,
-                                                 identifier1, identifier2, identifier3,
+                                                 identifier1, identifier2, identifier3, identifier4,
                                                  params, data)
 
-        def patch(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def patch(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_certauth('PATCH', self._parts,
-                                                 identifier1, identifier2, identifier3,
+                                                 identifier1, identifier2, identifier3, identifier4,
                                                  params, data)
 
-        def post(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None, files=None):
+        def post(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_certauth('POST', self._parts,
-                                                 identifier1, identifier2, identifier3,
+                                                 identifier1, identifier2, identifier3, identifier4,
                                                  params, data, files)
 
-        def put(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def put(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_certauth('PUT', self._parts,
-                                                 identifier1, identifier2, identifier3,
+                                                 identifier1, identifier2, identifier3, identifier4,
                                                  params, data)
 
-        def delete(self, identifier1=None, identifier2=None, identifier3=None, params=None, data=None):
+        def delete(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
             return self._base.call_with_certauth('DELETE', self._parts,
-                                                 identifier1, identifier2, identifier3,
+                                                 identifier1, identifier2, identifier3, identifier4,
                                                  params, data)
 
-    def add(self, t, p1, p2=None, p3=None):
+    def add(self, t, p1, p2=None, p3=None, p4=None, p5=None):
         """add api call to class"""
 
         a = []
@@ -851,12 +844,16 @@ class CloudFlare(object):
             a += p2.split('/')
         if p3:
             a += p3.split('/')
+        if p4:
+            a += p4.split('/')
+        if p5:
+            a += p5.split('/')
 
         branch = self
         for element in a[0:-1]:
             try:
                 if '-' in element:
-                    branch = getattr(element, element.replace('-','_'))
+                    branch = getattr(branch, element.replace('-','_'))
                 else:
                     branch = getattr(branch, element)
             except:
@@ -866,7 +863,7 @@ class CloudFlare(object):
 
         try:
             if '-' in name:
-                f = getattr(element, name.replace('-','_'))
+                f = getattr(branch, name.replace('-','_'))
             else:
                 f = getattr(branch, name)
             # already exists - don't let it overwrite
@@ -876,19 +873,22 @@ class CloudFlare(object):
             pass
 
         if t == 'VOID':
-            f = self._AddUnused(self._base, p1, p2, p3)
+            f = self._AddUnused(self._base, p1, p2, p3, p4, p5)
         elif t == 'OPEN':
-            f = self._AddNoAuth(self._base, p1, p2, p3)
+            f = self._AddNoAuth(self._base, p1, p2, p3, p4, p5)
         elif t == 'AUTH':
-            f = self._AddWithAuth(self._base, p1, p2, p3)
+            f = self._AddWithAuth(self._base, p1, p2, p3, p4, p5)
         elif t == 'CERT':
-            f = self._AddWithCertAuth(self._base, p1, p2, p3)
+            f = self._AddWithCertAuth(self._base, p1, p2, p3, p4, p5)
         elif t == 'AUTH_UNWRAPPED':
-            f = self._AddWithAuthUnwrapped(self._base, p1, p2, p3)
+            f = self._AddWithAuthUnwrapped(self._base, p1, p2, p3, p4, p5)
         else:
             # should never happen
             raise CloudFlareAPIError(0, 'api load type mismatch')
 
+        if keyword.iskeyword(name):
+            ## add an extra keyword postfix'ed with underscore so it can used with Python code
+            setattr(branch, name + '_', f)
         if '-' in name:
             # dashes (vs underscores) cause issues in Python and other languages
             setattr(branch, name.replace('-','_'), f)
@@ -913,9 +913,18 @@ class CloudFlare(object):
                 # it's a known api call - lets show the result and continue down the tree
                 if 'delete' in d or 'get' in d or 'patch' in d or 'post' in d or 'put' in d:
                     # only show the result if a call exists for this part
+                    if '_parts_unused' in d:
+                            # This is an uncallable endpoint - presently no way to return this info
+                            # w.append(str(a)[1:-1] + ' ; UNUSED')
+                            pass
                     if '_parts' in d:
-                        # handle underscores by returning the actual API call vs the method name
-                        w.append(str(a)[1:-1].replace('/:id/','/'))
+                        if n[-1] == '_' and keyword.iskeyword(n[:-1]):
+                            # remove the extra keyword postfix'ed with underscore
+                            pass
+                        else:
+                            # handle underscores by returning the actual API call vs the method name
+                            w.append(str(a)[1:-1])
+                            ## w.append(str(a)[1:-1].replace('/:id/','/'))
                 w = w + self.api_list(a, s + '/' + n)
         return w
 
@@ -929,8 +938,8 @@ class CloudFlare(object):
 
         try:
             config = read_configs(profile)
-        except:
-            raise CloudFlareAPIError(0, 'profile/configuration read error')
+        except Exception as e:
+            raise CloudFlareAPIError(0, str(e))
 
         # class creation values override all configuration values
         if email is not None:
